package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchTool;
import cn.estsh.i3plus.pojo.model.license.ImppLicense;
import cn.estsh.impp.framework.boot.license.ImppLicenseTool;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogException;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogOperate;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogSystem;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

/**
 * @Description : 后台服务
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-06-19 下午2:32
 * @Modify:
 **/
@RestController
@Api(tags = "后台运维服务")
@RequestMapping("/impp/white/backstage")
public class BackstageController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(BackstageController.class);

    @Autowired
    private ISystemInitService systemInitService;
    @Autowired
    private ISysLogService logService;


    @GetMapping(value = "/redis-put-all")
    @ApiOperation(value = "重新加载所有缓存数据",notes = "重新加载所有缓存数据")
    public ResultBean redisPutAll(){
        try {
            systemInitService.putAndLoadAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-language")
    @ApiOperation(value = "重新加载语言数据",notes = "重新加载语言数据")
    public ResultBean redisPutSystemLanguage(){
        try {
            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-config")
    @ApiOperation(value = "重新加载系统配置数据",notes = "重新加载系统配置数据")
    public ResultBean redisPutSystemConfig(){
        try {
            systemInitService.putAndLoadSysConfig();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-dictionary")
    @ApiOperation(value = "重新加载系统字典数据",notes = "重新加载系统字典数据")
    public ResultBean redisPutSystemDictionary(){
        try {
            systemInitService.putAndLoadSysDictionary();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/license")
    @ApiOperation(value = "重新加载授权数据",notes = "重新加载授权数据")
    public ResultBean updateLicense(ImppLicense license){
        try {
            ValidatorBean.beginValid(license)
                    .checkNotZero("licenseMaxOrganize",license.getLicenseMaxOrganize())
                    .checkNotZero("licenseMaxWarehouse",license.getLicenseMaxWarehouse())
                    .checkNotZero("licenseMaxProductionLine",license.getLicenseMaxProductionLine())
                    .checkNotZero("licenseMaxUser",license.getLicenseMaxUser())
                    .checkNotZero("licenseMaxUserSession",license.getLicenseMaxUserSession())
                    .notNull("licenseUpdateUserName",license.getLicenseUpdateUserName())
                    .notNull("licenseUpdateDateTime",license.getLicenseUpdateDateTime())
                    .notNull("licenseDescription",license.getLicenseDescription());

            ImppLicenseTool.updateImppLicense(license);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/clean-log")
    @ApiOperation(value = "添加操作日志", notes = "添加操作日志，可用于国际化")
    public ResultBean logClean(String time) {
        try{
            Date thisTime = TimeTool.parseDate(time);
            String startTime = TimeTool.getToday(TimeTool.getDateBefore(thisTime, 0)) + " 00:00:00";
            String endTime = TimeTool.getToday(TimeTool.getDateBefore(thisTime, 0)) + " 23:59:59";

            SysLogOperate operate = new SysLogOperate();
            operate.setCreateDatetime(startTime);
            operate.setCreateDateTimeStart(startTime);
            operate.setCreateDateTimeEnd(endTime);

            SysLogSystem system = new SysLogSystem();
            system.setCreateDatetime(startTime);
            system.setCreateDateTimeStart(startTime);
            system.setCreateDateTimeEnd(endTime);

            SysLogException exception = new SysLogException();
            exception.setCreateDatetime(startTime);
            exception.setCreateDateTimeStart(startTime);
            exception.setCreateDateTimeEnd(endTime);

            SysLogTaskTime taskTime = new SysLogTaskTime();
            taskTime.setCreateDatetime(startTime);
            taskTime.setCreateDateTimeStart(startTime);
            taskTime.setCreateDateTimeEnd(endTime);

            ElasticSearchTool.reload();
            logService.deleteLogOperate(operate);
            logService.deleteLogSystem(system);
            logService.deleteLogException(exception);
            logService.deleteLogTaskTime(taskTime);

//            mongoService.deleteLogOperate(operate);
//            mongoService.deleteLogSystem(system);
//            mongoService.deleteLogException(exception);
//            mongoService.deleteLogTaskTime(taskTime);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
